@@ -1,24 +1,14 @@
 /* srv.c - DNS SRV code
  * Copyright (C) 2003, 2009 Free Software Foundation, Inc.
  *
- * This file is part of GnuPG.
+ * This file is part of GNUPG.
  *
- * This file is free software; you can redistribute it and/or modify
- * it under the terms of either
+ * GNUPG is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *   - the GNU Lesser General Public License as published by the Free
- *     Software Foundation; either version 3 of the License, or (at
- *     your option) any later version.
- *
- * or
- *
- *   - the GNU General Public License as published by the Free
- *     Software Foundation; either version 2 of the License, or (at
- *     your option) any later version.
- *
- * or both in parallel, as here.
- *
- * This file is distributed in the hope that it will be useful,
+ * GNUPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -30,10 +20,7 @@
 #include <config.h>
 #include <sys/types.h>
 #ifdef _WIN32
-# ifdef HAVE_WINSOCK2_H
-#  include <winsock2.h>
-# endif
-# include <windows.h>
+#include <windows.h>
 #else
 #include <netinet/in.h>
 #include <arpa/nameser.h>
@@ -45,10 +32,12 @@
 #include <time.h>
 #ifdef USE_ADNS
 # include <adns.h>
+# ifndef HAVE_ADNS_FREE
+#  define adns_free free
+# endif
 #endif
 
 #include "util.h"
-#include "host2net.h"
 #include "srv.h"
 
 /* Not every installation has gotten around to supporting SRVs
@@ -83,7 +72,7 @@ getsrv (const char *name,struct srventry **list)
   {
     adns_state state;
     adns_answer *answer = NULL;
-
+    
     rc = adns_init (&state, adns_if_noerrprint, NULL);
     if (rc)
       {
@@ -99,12 +88,12 @@ getsrv (const char *name,struct srventry **list)
         adns_finish (state);
         return -1;
       }
-    if (answer->status != adns_s_ok
+    if (answer->status != adns_s_ok 
         || answer->type != adns_r_srv || !answer->nrrs)
       {
-        log_error ("DNS query returned an error or no records: %s (%s)\n",
-                   adns_strerror (answer->status),
-                   adns_errabbrev (answer->status));
+        /* log_error ("DNS query returned an error or no records: %s (%s)\n", */
+        /*            adns_strerror (answer->status), */
+        /*            adns_errabbrev (answer->status)); */
         adns_free (answer);
         adns_finish (state);
         return 0;
@@ -120,7 +109,7 @@ getsrv (const char *name,struct srventry **list)
             log_info ("hostname in SRV record too long - skipped\n");
             continue;
           }
-
+      
         newlist = xtryrealloc (*list, (srvcount+1)*sizeof(struct srventry));
         if (!newlist)
           goto fail;
@@ -128,7 +117,7 @@ getsrv (const char *name,struct srventry **list)
         memset (&(*list)[srvcount], 0, sizeof(struct srventry));
         srv = &(*list)[srvcount];
         srvcount++;
-
+      
         srv->priority = answer->rrs.srvha[count].priority;
         srv->weight   = answer->rrs.srvha[count].weight;
         srv->port     = answer->rrs.srvha[count].port;
@@ -145,29 +134,29 @@ getsrv (const char *name,struct srventry **list)
     unsigned char *pt, *emsg;
     int r;
     u16 dlen;
-
+    
     r = res_query (name, C_IN, T_SRV, answer, sizeof answer);
     if (r < sizeof (HEADER) || r > sizeof answer)
       return -1;
     if (header->rcode != NOERROR || !(count=ntohs (header->ancount)))
       return 0; /* Error or no record found.  */
-
+    
     emsg = &answer[r];
     pt = &answer[sizeof(HEADER)];
-
+  
     /* Skip over the query */
     rc = dn_skipname (pt, emsg);
     if (rc == -1)
       goto fail;
-
+  
     pt += rc + QFIXEDSZ;
-
+  
     while (count-- > 0 && pt < emsg)
       {
         struct srventry *srv=NULL;
         u16 type,class;
         struct srventry *newlist;
-
+      
         newlist = xtryrealloc (*list, (srvcount+1)*sizeof(struct srventry));
         if (!newlist)
           goto fail;
@@ -175,39 +164,38 @@ getsrv (const char *name,struct srventry **list)
         memset(&(*list)[srvcount],0,sizeof(struct srventry));
         srv=&(*list)[srvcount];
         srvcount++;
-
+      
         rc = dn_skipname(pt,emsg); /* the name we just queried for */
         if (rc == -1)
           goto fail;
         pt+=rc;
-
+      
         /* Truncated message? */
         if((emsg-pt)<16)
           goto fail;
-
-        type = buf16_to_u16 (pt);
-        pt += 2;
+      
+        type=*pt++ << 8;
+        type|=*pt++;
         /* We asked for SRV and got something else !? */
         if(type!=T_SRV)
           goto fail;
-
-        class = buf16_to_u16 (pt);
-        pt += 2;
+      
+        class=*pt++ << 8;
+        class|=*pt++;
         /* We asked for IN and got something else !? */
         if(class!=C_IN)
           goto fail;
-
-        pt += 4; /* ttl */
-        dlen = buf16_to_u16 (pt);
-        pt += 2;
-
-        srv->priority = buf16_to_ushort (pt);
-        pt += 2;
-        srv->weight = buf16_to_ushort (pt);
-        pt += 2;
-        srv->port = buf16_to_ushort (pt);
-        pt += 2;
-
+      
+        pt+=4; /* ttl */
+        dlen=*pt++ << 8;
+        dlen|=*pt++;
+        srv->priority=*pt++ << 8;
+        srv->priority|=*pt++;
+        srv->weight=*pt++ << 8;
+        srv->weight|=*pt++;
+        srv->port=*pt++ << 8;
+        srv->port|=*pt++;
+      
         /* Get the name.  2782 doesn't allow name compression, but
            dn_expand still works to pull the name out of the
            packet. */
@@ -227,17 +215,17 @@ getsrv (const char *name,struct srventry **list)
       }
   }
 #endif /*!USE_ADNS*/
-
+  
   /* Now we have an array of all the srv records. */
-
+  
   /* Order by priority */
   qsort(*list,srvcount,sizeof(struct srventry),priosort);
-
+  
   /* For each priority, move the zero-weighted items first. */
   for (i=0; i < srvcount; i++)
     {
       int j;
-
+      
       for (j=i;j < srvcount && (*list)[i].priority == (*list)[j].priority; j++)
         {
           if((*list)[j].weight==0)
@@ -246,12 +234,12 @@ getsrv (const char *name,struct srventry **list)
               if(j!=i)
                 {
                   struct srventry temp;
-
+                  
                   memcpy (&temp,&(*list)[j],sizeof(struct srventry));
                   memcpy (&(*list)[j],&(*list)[i],sizeof(struct srventry));
                   memcpy (&(*list)[i],&temp,sizeof(struct srventry));
                 }
-
+              
               break;
             }
         }
@@ -267,15 +255,15 @@ getsrv (const char *name,struct srventry **list)
     {
       int j;
       float prio_count=0,chose;
-
+      
       for (j=i; j < srvcount && (*list)[i].priority == (*list)[j].priority; j++)
         {
           prio_count+=(*list)[j].weight;
           (*list)[j].run_count=prio_count;
         }
-
+      
       chose=prio_count*rand()/RAND_MAX;
-
+      
       for (j=i;j<srvcount && (*list)[i].priority==(*list)[j].priority;j++)
         {
           if (chose<=(*list)[j].run_count)
@@ -284,7 +272,7 @@ getsrv (const char *name,struct srventry **list)
               if(j!=i)
                 {
                   struct srventry temp;
-
+                  
                   memcpy(&temp,&(*list)[j],sizeof(struct srventry));
                   memcpy(&(*list)[j],&(*list)[i],sizeof(struct srventry));
                   memcpy(&(*list)[i],&temp,sizeof(struct srventry));
@@ -293,7 +281,7 @@ getsrv (const char *name,struct srventry **list)
             }
         }
     }
-
+  
   return srvcount;
 
  fail:
@@ -328,6 +316,6 @@ main(int argc,char *argv[])
 
 /*
 Local Variables:
-compile-command: "cc -DTEST -I.. -I../include -Wall -g -o srv srv.c -lresolv  ../tools/no-libgcrypt.o  ../common/libcommon.a"
+compile-command: "cc -DTEST -I.. -I../include -Wall -g -o srv srv.c -lresolv  ../tools/no-libgcrypt.o  ../jnlib/libjnlib.a"
 End:
 */
