@@ -36,7 +36,10 @@
 #include <stdlib.h>
 
 #ifdef HAVE_W32_SYSTEM
-#include <windows.h>
+# ifdef HAVE_WINSOCK2_H
+#  include <winsock2.h>
+# endif
+# include <windows.h>
 #endif
 
 #ifdef HAVE_LIBCURL
@@ -44,6 +47,7 @@
 #else
 #include "curl-shim.h"
 #endif
+#include "util.h"
 #include "keyserver.h"
 #include "ksutil.h"
 
@@ -77,7 +81,7 @@ register_timeout(void)
   sigemptyset(&act.sa_mask);
   act.sa_flags=0;
   return sigaction(SIGALRM,&act,NULL);
-#else 
+#else
   if(signal(SIGALRM,catch_alarm)==SIG_ERR)
     return -1;
   else
@@ -325,7 +329,7 @@ parse_ks_options(char *line,struct ks_options *opt)
 	  else if(start[12]=='=')
 	    {
 	      free(opt->ca_cert_file);
-	      opt->ca_cert_file=strdup(&start[13]);
+	      opt->ca_cert_file = make_filename_try (start+13, NULL);
 	      if(!opt->ca_cert_file)
 		return KEYSERVER_NO_MEMORY;
 	    }
@@ -596,7 +600,7 @@ ks_hextobyte (const char *s)
 
 
 /* Non localized version of toupper.  */
-int 
+int
 ks_toupper (int c)
 {
   if (c >= 'a' && c <= 'z')
